
//          Copyright Danny Havenith 2009.
// Distributed under the Boost Software License, Version 1.0.
//    (See accompanying file LICENSE_1_0.txt or copy at
//          http://www.boost.org/LICENSE_1_0.txt)

#if !defined( PACKAGER_HPP)
#define PACKAGER_HPP
#include <vector>
#include <algorithm> // for for_each
#include <boost/crc.hpp>
#include "packet_protocol.hpp"

//
// a packager splits a string (of 'bytes') into packets
// objects of this class will calculate a checksum and then send
// the packets to some packet-sink.
//
struct packager
{
	packager( packet_sink &sink_, int address_ = 0)
		:sink( sink_), address( address_)
	{
	}

	//
	// translate a string into a stream of packets and send those packets
	// to the sink.
	//
	void send_string( const std::string text)
	{
		packet_vector result;
		chop_string( text, result);
		sink.send( result);
	}

	void cursor_home()
	{
		send_string( "\015");
	}

private:
	// zero-initialized ccitt crc type
	typedef boost::crc_optimal<16, 0x1021, 0x0000, 0, false, false> ccitt_zero;
	typedef std::vector< packet> packet_vector;

	packet_sink &sink;
	char		address;
	static const int max_payload_size = 16;

	//
	// given a string that must fit in a packet, create a packet with that string
	// in the payload and calculate a checksum.
	//
	void package_string( const std::string &chunk, packet_vector &result)
	{
		packet p;

		p.address = address;
		p.payload = chunk;
		// notice the extra parentheses at the end of the expression.
		p.checksum = for_each( p.payload.begin(), p.payload.end(), ccitt_zero() )();
		p.size = (p.payload.length() - 1) & 0x0f;

		result.push_back( p);
	}

	//
	// chop a string into pieces that each fit in a single packet and add
	// those packets to the vector.
	//
	void chop_string( const std::string &text, packet_vector &result)
	{
		using std::string;

		string::const_iterator begin = text.begin();
		string::const_iterator end = 
			(text.end() - begin >= max_payload_size)?
				begin + max_payload_size:
				text.end();

		while (begin != end)
		{
			package_string( string( begin, end), result);
			begin = end;
			end = (text.end() - begin >= max_payload_size)?
						begin + max_payload_size:
						text.end();
		}
	} 

};

#endif //PACKAGER_HPP
